@@ -1,9 +1,3 @@
-/* 
- * $TSUKUBA_Release: Omni OpenMP Compiler 3 $
- * $TSUKUBA_Copyright:
- *  PLEASE DESCRIBE LICENSE AGREEMENT HERE
- *  $
- */
 package exc.block;
 
 import xcodeml.util.XmLog;
@@ -74,6 +68,12 @@ public class Bcons
             return new CompoundBlock(Xcode.F_STATEMENT_LIST, b_list);
     }
 
+    /** create compound statement (or statement list, block statement) block */
+    public static Block COMPOUND(BlockList b_list, Xcode code)
+    {
+        return new CompoundBlock(code, b_list);
+    }
+
     /** create 'pragma' statement block */
     public static Block PRAGMA(Xcode code, String pragma, Xobject args, BlockList body)
     {
@@ -110,11 +110,11 @@ public class Bcons
     }
     
     /** create Fortran 'where' statement block */
-    public static Block Fwhere(BasicBlock cond, BlockList then_part, BlockList else_part)
+    public static Block Fwhere(BasicBlock cond, BlockList then_part, BlockList else_part, String construct_name)
     {
-        return IF(Xcode.F_WHERE_STATEMENT, cond, then_part, else_part, null);
+        return IF(Xcode.F_WHERE_STATEMENT, cond, then_part, else_part, construct_name);
     }
-    
+
     /** create 'for' statement block */
     public static Block FOR(BasicBlock init, BasicBlock cond, BasicBlock iter, BlockList body, String construct_name)
     {
@@ -140,6 +140,24 @@ public class Bcons
         return FOR(init, cond, iter, body, null);
     }
 
+    /** create 'forall' statement block */
+    public static Block FforAll(Xobject v) // (construct_name or null) (VAR INDEX_RANGE)+ [condition] body
+    {
+        String construct_name = (v.getArgOrNull(0) != null) ? v.getArgOrNull(0).getName() : null;
+        XobjList ind_var_range = new XobjList();
+        int idx;
+        for (idx = 1; v.getArg(idx).Opcode() == Xcode.VAR; idx = idx + 2)
+            if (v.getArg(idx + 1).Opcode() == Xcode.F_INDEX_RANGE)
+                ind_var_range.add(new XobjList(null, v.getArg(idx), v.getArg(idx + 1)));
+            else
+                break;
+        BasicBlock cond = null;
+        if (v.getArg(idx).Opcode() != Xcode.F_STATEMENT_LIST)
+            cond = BasicBlock.Cond(v.getArg(idx++));
+        BlockList body = buildList(v.getArg(idx));
+        return new FforAllBlock(v.Type(), cond, ind_var_range, body, construct_name);
+    }
+
     /** create 'for' statement block */
     public static Block FORall(Xobject ind_var, Xobject lb, Xobject ub, Xobject step,
 			       Xcode checkOp, BlockList body)
@@ -151,9 +169,15 @@ public class Bcons
     }
     
     /** create Fortran 'do' statement block */
-    public static Block Fdo(Xobject var, Xobject idx_range, BlockList body, String construct_name)
+    public static Block Fdo(                  Xobject var, Xobject idx_range, BlockList body, String construct_name)
     {
-        return new FdoBlock(var, idx_range, body, construct_name);
+        return new FdoBlock(    null, var, idx_range, body, construct_name);
+    }
+
+    /** create Fortran 'do' statement block from Xobject */
+    public static Block Fdo(Xobject fdo_stmt)
+    {
+        return new FdoBlock(fdo_stmt.getLineNo(), fdo_stmt.getArg(1), fdo_stmt.getArg(2), buildList(fdo_stmt.getArg(3)), getArg0Name(fdo_stmt));
     }
 
     /** create 'while' statement block */
@@ -303,10 +327,10 @@ public class Bcons
     /** create FuctionBlock for XobjectDef */
     public static FunctionBlock buildFunctionBlock(XobjectDef d)
     {
-	return new FunctionBlock(d.getDef().Opcode(),d.getNameObj(),
+	return new FunctionBlock(d.getDef().getLineNo(),d.getDef().Opcode(),d.getNameObj(),
 				 d.getFuncIdList(), d.getFuncDecls(),
 				 buildBlock(d.getFuncBody()), 
-				 d.getFuncGccAttributes(), d.getParentEnv());
+				 d.getFuncGccAttributes(), d.getParentEnv(), d.getParentNameObj());
     }
 
     /** create block list for Xobject */
@@ -316,11 +340,20 @@ public class Bcons
         BlockList b_list = new BlockList();
         if(v == null)
             return b_list;
+        b_list.code = v.Opcode();
         
         switch(v.Opcode()) {
         case LIST: /* (LIST statement ....) */
         case F_STATEMENT_LIST: /* (F_STATEMENT_LIST statement ....) */
             break;
+        case F_BLOCK_STATEMENT:
+            /* (F_BLOCK_STATEMENT id-list decl statement-list) */
+            b_list.block_name = (XobjString)v.getArg(0);
+            b_list.id_list = v.getArg(1);
+            b_list.decls = v.getArg(2);
+            b_list.initLocalCoarrays((XobjList)v.getArgOrNull(4));
+            v = v.getArg(3);
+            break;
         case COMPOUND_STATEMENT:
             /* (COMPOUND_STATEMENT id-list decl statement-list) */
             b_list.id_list = v.getArg(0);
@@ -400,7 +433,10 @@ public class Bcons
 
         case F_STATEMENT_LIST:
         case COMPOUND_STATEMENT:
-            return COMPOUND(buildList(v));
+        case F_BLOCK_STATEMENT:
+            CompoundBlock cb = (CompoundBlock)COMPOUND(buildList(v), code);
+            cb.setLineNo(v.getLineNo());
+            return cb;
             
         case OMP_PRAGMA:
             return PRAGMA(Xcode.OMP_PRAGMA, v.getArg(0).getString(), v.getArgOrNull(1),
@@ -418,6 +454,9 @@ public class Bcons
             return IF(BasicBlock.Cond(v.getArg(0)), buildList(v.getArg(1)),
 		      buildList(v.getArg(2)));
             
+        case F_FORALL_STATEMENT: /* (FORALL construct_name index_range cond body) */
+            return FforAll(v);
+
         case FOR_STATEMENT: /* (FOR init cond iter body) */
             return FOR(BasicBlock.Statement(v.getArg(0)), BasicBlock.Cond(v.getArg(1)),
 		       BasicBlock.Statement(v.getArg(2)), buildList(v.getArg(3)));
@@ -463,7 +502,8 @@ public class Bcons
         case PRAGMA_LINE:
 	    return PRAGMA(Xcode.PRAGMA_LINE, v.getArg(0).getString(), null, null);
         case TEXT:
-            return new SimpleBlock(statement_list_code(), BasicBlock.Statement(v));
+	case LINEMARKER:
+	    return new SimpleBlock(statement_list_code(), BasicBlock.Statement(v));
             
         case F_IF_STATEMENT: /* (F_IF_STATMENT construct_name cond then-part else-part) */
             return IF(code, BasicBlock.Cond(v.getArg(1)),
@@ -471,10 +511,10 @@ public class Bcons
             
         case F_WHERE_STATEMENT: /* (F_WHERE_STATMENT () cond then-part else-part) */
             return Fwhere(BasicBlock.Cond(v.getArg(1)),
-			  buildList(v.getArg(2)), buildList(v.getArg(3)));
+			  buildList(v.getArg(2)), buildList(v.getArg(3)), getArg0Name(v));
             
         case F_DO_STATEMENT: /* (F_DO_STATEMENT construct_name var index_range body) */
-            return Fdo(v.getArg(1), v.getArg(2), buildList(v.getArg(3)), getArg0Name(v));
+            return Fdo(v);
             
         case F_DO_WHILE_STATEMENT: /* (F_DO_WHILE_STATEMENT construct_name cond */
             return WHILE(code, BasicBlock.Cond(v.getArg(1)),
