@@ -1,33 +1,35 @@
-/* 
- * $TSUKUBA_Release: Omni OpenMP Compiler 3 $
- * $TSUKUBA_Copyright:
- *  PLEASE DESCRIBE LICENSE AGREEMENT HERE
- *  $
- */
 package exc.block;
 
+import java.util.Vector;
+
 /**
  * super class for Block iterator
  */
 public abstract class BlockIterator
 {
+    Vector<Block> blocks;
+    int index;
+
     Block currentBlock = null;
 
     /** constructor with block */
     public BlockIterator(Block b)
     {
         currentBlock = b; // set defaults
+        init(b);
     }
 
     public BlockIterator(BlockList body)
     {
-        // nothing
+        init(body);
     }
 
     public abstract void init();
 
     public abstract void init(Block b);
 
+    public abstract void init(BlockList body);
+
     public abstract void next();
 
     public abstract boolean end();
@@ -45,4 +47,21 @@ public abstract class BlockIterator
     {
         currentBlock.replace(b);
     }
+
+    public void setAside(Block b)
+    {
+        if (blocks.removeElement(b))
+            index = blocks.lastIndexOf(currentBlock);
+    }
+
+    public void setAside(Vector<Block> vb)
+    {
+        if (blocks.removeAll(vb))
+            index = blocks.lastIndexOf(currentBlock);
+    }
+
+    public Vector<Block> getContainer()
+    {
+        return blocks;
+    }
 }
