@@ -1,18 +1,11 @@
-/* 
- * $TSUKUBA_Release: Omni OpenMP Compiler 3 $
- * $TSUKUBA_Copyright:
- *  PLEASE DESCRIBE LICENSE AGREEMENT HERE
- *  $
- */
 package exc.object;
 
 import exc.block.Block;
 import exc.util.XobjectVisitable;
 import exc.util.XobjectVisitor;
-import xcodeml.IXobject;
-import xcodeml.XmException;
+import xcodeml.util.XmException;
 import xcodeml.util.XmOption;
-
+import xcodeml.util.IXobject;
 
 /**
  * Abstract class of expression tree objects in Xobject.
@@ -550,7 +543,6 @@ public class Xobject extends PropObject implements IXobject, XobjectVisitable, I
         return o;
     }
 
-
     /**
      * 
      * Dummy method at base class, it causes exception.
@@ -970,4 +962,9 @@ public class Xobject extends PropObject implements IXobject, XobjectVisitable, I
         return ((XobjList)this).hasNullArg();
       return false;
     }
+
+    public boolean isIndexRange()
+    {
+      return code == Xcode.INDEX_RANGE;
+    }
 }
