@@ -1,27 +1,3 @@
-/*
- * $TSUKUBA_Release: Omni Compiler Version 0.9.0 $
- * $TSUKUBA_Copyright:
- *  Copyright (C) 2010-2014 University of Tsukuba, 
- *  	      2012-2014  University of Tsukuba and Riken AICS
- *  
- *  This software is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU Lesser General Public License version
- *  2.1 published by the Free Software Foundation.
- *  
- *  Please check the Copyright and License information in the files named
- *  COPYRIGHT and LICENSE under the top  directory of the Omni Compiler
- *  Software release kit.
- *  
- *  * The specification of XcalableMP has been designed by the XcalableMP
- *    Specification Working Group (http://www.xcalablemp.org/).
- *  
- *  * The development of this software was partially supported by "Seamless and
- *    Highly-productive Parallel Programming Environment for
- *    High-performance computing" project funded by Ministry of Education,
- *    Culture, Sports, Science and Technology, Japan.
- *  $
- */
-
 package exc.xcalablemp;
 
 import exc.object.*;
@@ -678,7 +654,7 @@ public class XMPgpuDecompileWriter extends PrintWriter {
   private String makeDeclType(Xtype type, String name) {
     String decltype = "";
     Xtype t;
-    Stack nested_decls = new Stack();
+    Stack<Xtype> nested_decls = new Stack<Xtype>();
 
     for (t = type; t != null; t = t.getRef()) {
       if (t.getKind() == Xtype.POINTER ||
@@ -766,7 +742,7 @@ public class XMPgpuDecompileWriter extends PrintWriter {
           break;
         }
 
-        if (t.getArrayDim() < 0) {
+        if (t.getArraySize() < 0) {
           decltype += "(";
           decltype += "*";
           decltype += makeDeclaratorRec(decls, name);
@@ -784,10 +760,10 @@ public class XMPgpuDecompileWriter extends PrintWriter {
           decltype += ")";
         }
 
-        if (t.getArrayDim() <= 0) {
+        if (t.getArraySize() <= 0) {
           decltype += "[]";
         } else {
-          decltype += "[" + t.getArrayDim() + "]";
+          decltype += "[" + t.getArraySize() + "]";
         }
       } break;
     default:
@@ -960,14 +936,14 @@ public class XMPgpuDecompileWriter extends PrintWriter {
           } break;
         case Xtype.ENUM:
           {
-            if(type.getMoeList() == null) {
+            if(type.getMemberList() == null) {
               break;
             }
 
             printTypeName(type);
             println(" {");
 
-            for (XobjArgs a1 = type.getMoeList().getArgs(); a1 != null; a1=a1.nextArgs()) {
+            for (XobjArgs a1 = type.getMemberList().getArgs(); a1 != null; a1=a1.nextArgs()) {
               if ((id = findIdent(id_list, a1.getArg())) == null) {
                 fatal("Enum ID is not found");
               }
