@@ -891,6 +891,101 @@ type_parameter_expv_equals(expv v1, expv v2, int is_strict, int for_argument, in
             return FALSE;
         }
     }
+
+    if(EXPR_CODE(v1) == FUNCTION_CALL && EXPR_CODE(v2) == FUNCTION_CALL) {
+        char *name1 = NULL;
+        char *name2 = NULL;
+
+        SYMBOL s1 = EXPV_NAME(EXPR_ARG1(v1));
+        ID fId1 = find_ident(s1);
+        if(fId1 != NULL && PROC_CLASS(fId1) == P_INTRINSIC){
+            name1 = SYM_NAME(ID_SYM(fId1));
+        } else if(SYM_TYPE(s1) == S_INTR){
+            name1 = SYM_NAME(s1);
+        }
+
+        SYMBOL s2 = EXPV_NAME(EXPR_ARG1(v2));
+        ID fId2 = find_ident(s2);
+        if(fId2 != NULL && PROC_CLASS(fId2) == P_INTRINSIC){
+            name2 = SYM_NAME(ID_SYM(fId2));
+        } else if(SYM_TYPE(s2) == S_INTR){
+            name2 = SYM_NAME(s2);
+        }
+
+        if(strcmp(name1, name2) == 0) {
+            // Check the value of selected_real_kind
+            if(strcmp(name1, "selected_real_kind") == 0) {
+                if(!EXPR_HAS_ARG1(EXPR_ARG2(v1))
+                    && !EXPR_HAS_ARG1(EXPR_ARG2(v2)))
+                {
+                    // both selected_real_kind() have no arg.
+                    return TRUE;
+                } else if (!EXPR_HAS_ARG1(EXPR_ARG2(v1))
+                    || !EXPR_HAS_ARG1(EXPR_ARG2(v2)))
+                {
+                    // one of selected_real_kind() has no arg and one has.
+                    return FALSE;
+                } else {
+                    // both selected real kind have at least one args
+                    int nbArgs1 = EXPR_HAS_ARG3(EXPR_ARG2(v1)) ?
+                        3 : EXPR_HAS_ARG2(EXPR_ARG2(v1)) ? 2 : 1;
+                    int nbArgs2 = EXPR_HAS_ARG3(EXPR_ARG2(v2)) ?
+                        3 : EXPR_HAS_ARG2(EXPR_ARG2(v2)) ? 2 : 1;
+
+                    if(nbArgs1 != nbArgs2) {
+                        return FALSE;
+                    } else {
+                        // check args for compatibility
+                        if(nbArgs1 == 3
+                            && EXPV_INT_VALUE(EXPR_ARG1(EXPR_ARG2(v1))) ==
+                            EXPV_INT_VALUE(EXPR_ARG1(EXPR_ARG2(v2)))
+                            && EXPV_INT_VALUE(EXPR_ARG2(EXPR_ARG2(v1))) ==
+                            EXPV_INT_VALUE(EXPR_ARG2(EXPR_ARG2(v2)))
+                            && EXPV_INT_VALUE(EXPR_ARG3(EXPR_ARG2(v1))) ==
+                            EXPV_INT_VALUE(EXPR_ARG3(EXPR_ARG2(v2))))
+                        {
+                            return TRUE;
+                        } else if(nbArgs1 == 2
+                            && EXPV_INT_VALUE(EXPR_ARG1(EXPR_ARG2(v1))) ==
+                            EXPV_INT_VALUE(EXPR_ARG1(EXPR_ARG2(v2)))
+                            && EXPV_INT_VALUE(EXPR_ARG2(EXPR_ARG2(v1))) ==
+                            EXPV_INT_VALUE(EXPR_ARG2(EXPR_ARG2(v2))))
+                        {
+                            return TRUE;
+                        } else if(nbArgs1 == 1
+                            && EXPV_INT_VALUE(EXPR_ARG1(EXPR_ARG2(v1))) ==
+                            EXPV_INT_VALUE(EXPR_ARG1(EXPR_ARG2(v2))))
+                        {
+                            return TRUE;
+                        } else {
+                            return FALSE;
+                        }
+                    }
+                }
+            } else if(strcmp(name1, "selected_int_kind") == 0) {
+                // Check compatiblity of selected_int_kind(R)
+                if(EXPR_ARG2(v1) == NULL && EXPR_ARG2(v2) == NULL) {
+                    return TRUE;
+                } else if(EXPV_INT_VALUE(EXPR_ARG1(EXPR_ARG2(v1))) ==
+                    EXPV_INT_VALUE(EXPR_ARG1(EXPR_ARG2(v2))))
+                {
+                    return TRUE;
+                } else {
+                    return FALSE;
+                }
+            }
+        } else {
+            return FALSE;
+        }
+    } else if(EXPR_CODE(v1) == F_VAR && EXPR_CODE(v2) == F_VAR) {
+        // Both kind are variables
+        if(EXPV_NAME(v1) != NULL && EXPV_NAME(v2) != NULL 
+            && strcmp(SYM_NAME(EXPV_NAME(v1)), SYM_NAME(EXPV_NAME(v2))) == 0) 
+        {
+            return TRUE;
+        }
+        return FALSE;
+    }
     /* CANNOT RECOGNIZE THE VALUE OF EXPV, pass */
     return TRUE;
 }
@@ -1790,7 +1885,7 @@ function_type_is_appliable(TYPE_DESC ftp, expv actual_args, int issue_error)
 int
 struct_type_is_compatible_for_assignment(TYPE_DESC tp1, TYPE_DESC tp2, int is_pointer_set)
 {
-    TYPE_DESC btp2;
+    TYPE_DESC btp1, btp2;
 
     assert(tp1 != NULL && TYPE_BASIC_TYPE(tp1) == TYPE_STRUCT);
 
@@ -1827,13 +1922,21 @@ struct_type_is_compatible_for_assignment(TYPE_DESC tp1, TYPE_DESC tp2, int is_po
     }
     if (debug_flag) fprintf(debug_fp," not match\n");
 
+    btp1 = getBaseParameterizedType(tp1);
     btp2 = getBaseParameterizedType(tp2);
 
     if (debug_flag) fprintf(debug_fp,"* compare type names\n");
     if (!compare_derived_type_name(tp1, tp2)) {
         if (debug_flag) fprintf(debug_fp,"                                      ... not match\n");
-
-        if (TYPE_IS_CLASS(tp1) && TYPE_PARENT(btp2) && is_pointer_set) {
+        /* Check if the imported id match. Type ID might be not identical 
+         * in case of multiple indirection in module hierachy. */
+        if(btp1->imported_id && btp2->imported_id 
+            && strcmp(btp1->imported_id, btp2->imported_id) == 0) 
+        {
+            return TRUE;
+        } else if ((TYPE_IS_CLASS(tp1) || TYPE_IS_POINTER(tp1)) 
+            && TYPE_PARENT(btp2) && is_pointer_set) 
+        {
             if (debug_flag) fprintf(debug_fp,"* compare PARENT type\n");
             return struct_type_is_compatible_for_assignment(tp1, TYPE_PARENT_TYPE(btp2), is_pointer_set);
         } else {
@@ -2171,7 +2274,7 @@ get_binary_comparative_intrinsic_operation_type(TYPE_DESC t0, TYPE_DESC t1) {
 
     }
 
-    return (isValid = TRUE) ? type_basic(TYPE_LOGICAL) : NULL;
+    return (isValid == TRUE) ? type_basic(TYPE_LOGICAL) : NULL;
 }
 
 TYPE_DESC
@@ -2239,7 +2342,7 @@ get_binary_equal_intrinsic_operation_type(TYPE_DESC t0, TYPE_DESC t1) {
 
     }
 
-    return (isValid = TRUE) ? type_basic(TYPE_LOGICAL) : NULL;
+    return (isValid == TRUE) ? type_basic(TYPE_LOGICAL) : NULL;
 }
 
 int
