@@ -98,16 +98,19 @@ typedef struct type_descriptor
     struct type_descriptor *ref;         /* reference to other */
     struct ident_descriptor *tagname;    /* derived type tagname */
     char is_referenced;
+    char* imported_id;         /* original imported id, if imported */
     expv kind;                 /* kind parameter */
     expv leng;                 /* len parameter */
     int size;                  /* for TYPE_CHAR char length */
     int is_declared;           /* boolean for type has declared.
                                   (only used by struct type) */
+    int is_imported;           /* type was imported from xmod */
     int is_modified;           /* modified with VOLATILE or ASYNCHRONOUS */
     expv bind_name;            /* ISO BIND C name attribute */
 
     int is_access_inferred;    /* boolean flag that tell if the access-spec is
                                   inferred or clearly stated */
+    int is_assumed_type;       /* TYPE(*) */                                     
     struct type_attr {
 #define TYPE_ATTR_PARAMETER         0x00000001
 #define TYPE_ATTR_ALLOCATABLE       0x00000002
@@ -216,6 +219,9 @@ extern TYPE_DESC basic_type_desc[];
 #define TYPE_REF(tp)            ((tp)->ref)
 #define TYPE_TAGNAME(tp)        ((tp)->tagname)
 #define TYPE_IS_REFERENCED(tp)  ((tp)->is_referenced)
+#define TYPE_IS_ASSUMED(tp)     ((tp)->is_assumed_type)
+#define TYPE_SET_ASSUMED(tp)    ((tp)->is_assumed_type = TRUE)
+#define TYPE_UNSET_ASSUMED(tp)  ((tp)->is_assumed_type = FALSE)
 #define TYPE_CODIMENSION(tp)    ((tp)->codims)
 #define TYPE_LINK_ADD(tp, tlist, ttail) \
     { if((tlist) == NULL) (tlist) = (tp); \
@@ -437,6 +443,8 @@ extern TYPE_DESC basic_type_desc[];
 
 #define TYPE_IS_RESHAPED(tp)            ((tp)->is_reshaped_type)
 
+#define TYPE_IS_IMPORTED(tp) ((tp)->is_imported)
+
 #define TYPE_IS_ARRAY_ASSUMED_SIZE(tp) \
                 (TYPE_ARRAY_ASSUME_KIND(tp) == ASSUMED_SIZE)
 #define TYPE_IS_ARRAY_ASSUMED_SHAPE(tp) \
