@@ -47,6 +47,33 @@ void remove_sp_list(SP_LIST *list)
     list->err_no = 0;
 }
 
+/* Check potential flagged undefined variable that have been switch to proc */
+void sp_check(ID id) {
+    SP_LIST *list = sp_list_head;
+    SP_LIST *prev = NULL;
+    while(list) {
+        SP_LIST *next = list->next;
+        //printf("item %s\n", ID_NAME(list->info.id));
+        if(list->err_no == SP_ERR_UNDEF_TYPE_VAR) {
+            // ID was flagged, now proc so error can be removed
+            if(strcmp(ID_NAME(id), ID_NAME(list->info.id)) == 0) {
+              if(list == sp_list_head) {
+                // Item is the head. Point head to next element.
+                list->next = NULL;
+                sp_list_head = next;
+                free((void*)list);
+              } else {
+                // Item is in middle of the list or tail
+                prev->next = next;
+                free((void*)list);
+              }
+            }
+        }
+        prev = list;
+        list = next;
+    }
+}
+
 void sp_link_id(ID id, int err_no, lineno_info *line)
 {
   SP_LIST *list;
@@ -105,8 +132,12 @@ static int second_pass_clean()
     /* error */
     switch(list->err_no){
     case SP_ERR_UNDEF_TYPE_VAR: /* 1 */
-      current_line = list->line;
-      error("attempt to use undefined type variable, %s", ID_NAME(list->info.id));
+    case SP_ERR_FOWARD_FUNC: /* 5 */
+     current_line = list->line;
+      TYPE_DESC tpFunc = list->info.id->type;
+      if (tpFunc && (IS_SUBR(tpFunc) || IS_FUNCTION_TYPE(tpFunc))) break;
+      error("attempt to use undefined variable or function, %s",
+        ID_NAME(list->info.id));
       err_num++;
       break;
     case SP_ERR_CHAR_LEN: /* 2*/
@@ -416,7 +447,13 @@ static void second_pass_expv_scan(expv v)
   case F_LT_EXPR:
   case F_LE_EXPR:
   case F_NE_EXPR:
-  case F_OR_EXPR:
+  case F_EQ_EXPR_DOT:
+  case F_GT_EXPR_DOT:
+  case F_GE_EXPR_DOT:
+  case F_LT_EXPR_DOT:
+  case F_LE_EXPR_DOT:
+  case F_NE_EXPR_DOT:
+  case F_OR_EXPR_DOT:
   case F_AND_EXPR:
   case F_NOT_EXPR:
   case F_ARRAY_REF:
@@ -485,6 +522,12 @@ static void second_pass_expv_scan(expv v)
   case F95_LEOP:
   case F95_GEOP:
   case F95_GTOP:
+  case F95_EQOP_DOT:
+  case F95_NEOP_DOT:
+  case F95_LTOP_DOT:
+  case F95_LEOP_DOT:
+  case F95_GEOP_DOT:
+  case F95_GTOP_DOT:
   case F95_NOTOP:
   case F95_ANDOP:
   case F95_OROP:
